@@ -203,6 +203,7 @@ const char *GetXaLibPath();
 #define	BIT_CVT_NULL_DATE			(1L << 3)
 #define	BIT_ACCESSIBLE_ONLY			(1L << 4)
 #define	BIT_IGNORE_ROUND_TRIP_TIME		(1L << 5)
+#define	BIT_DISABLE_KEEPALIVE			(1L << 6)
 
 /*	Connection Defaults */
 #define DEFAULT_PORT				"5432"
@@ -242,7 +243,7 @@ const char *GetXaLibPath();
 #endif	/* WIN32 */
 #define DEFAULT_INT8AS			0
 #define DEFAULT_BYTEAASLONGVARBINARY	0
-#define DEFAULT_USESERVERSIDEPREPARE	0
+#define DEFAULT_USESERVERSIDEPREPARE	1
 #define DEFAULT_LOWERCASEIDENTIFIER	0
 #define DEFAULT_SSLMODE			SSLMODE_DISABLE
 #define DEFAULT_GSSAUTHUSEGSSAPI	0
@@ -296,7 +297,7 @@ int     changeDriverNameOfaDSN(const char *dsn, const char *driver_name, DWORD *
 UInt4	getExtraOptions(const ConnInfo *);
 BOOL	setExtraOptions(ConnInfo *, const char *str, const char *format);
 char	*extract_attribute_setting(const char *str, const char *attr, BOOL ref_comment);
-char	*extract_extra_attribute_setting(const char *str, const char *attr);
+char	*extract_extra_attribute_setting(const pgNAME setting, const char *attr);
 
 #ifdef	__cplusplus
 }
