@@ -16,6 +16,7 @@
 #include "psqlodbc.h"
 #include <string.h>
 
+#include "misc.h"
 #include "environ.h"
 #include "connection.h"
 #include "statement.h"
@@ -138,7 +139,7 @@ set_statement_option(ConnectionClass *conn,
 				stmt->options_orig.keyset_size = vParam;
 				if (!SC_get_Result(stmt)) 
 					stmt->options.keyset_size = vParam;
-				if (stmt->options.keyset_size != (int)vParam)
+				if (stmt->options.keyset_size != (SQLLEN) vParam)
 					changed = TRUE;
 			}
 
@@ -153,7 +154,7 @@ set_statement_option(ConnectionClass *conn,
 				stmt->options_orig.maxLength = vParam;
 				if (!SC_get_Result(stmt)) 
 					stmt->options.maxLength = vParam;
-				if (stmt->options.maxLength != (int)vParam)
+				if (stmt->options.maxLength != (SQLLEN) vParam)
 					changed = TRUE;
 			}
 			break;
@@ -167,7 +168,7 @@ set_statement_option(ConnectionClass *conn,
 				stmt->options_orig.maxRows = vParam;
 				if (!SC_get_Result(stmt)) 
 					stmt->options.maxRows = vParam;
-				if (stmt->options.maxRows != (int)vParam)
+				if (stmt->options.maxRows != (SQLLEN)vParam)
 					changed = TRUE;
 			}
 			break;
@@ -260,7 +261,7 @@ set_statement_option(ConnectionClass *conn,
 
 				if (stmt)
 				{
-					SC_set_error(stmt, STMT_NOT_IMPLEMENTED_ERROR, "Unknown statement option (Set)", NULL);
+					SC_set_error(stmt, STMT_NOT_IMPLEMENTED_ERROR, "Unknown statement option (Set)", func);
 					sprintf(option, "fOption=%d, vParam=" FORMAT_ULEN, fOption, vParam);
 					SC_log_error(func, option, stmt);
 				}
@@ -386,10 +387,10 @@ PGAPI_SetConnectOption(
 			mylog("%s: AUTOCOMMIT: transact_status=%d, vparam=" FORMAT_LEN "\n", func, conn->transact_status, vParam);
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
-			if (NULL != conn->asdum)
+			if (CC_is_in_global_trans(conn))
 			{
-				mylog("%s: Ignored AUTOCOMMIT in a distributed transaction, OK ?");
-				break;
+				CC_set_error(conn, CONN_TRANSACT_IN_PROGRES, "Don't change AUTOCOMMIT mode in a distributed transaction", func);
+				return SQL_ERROR;
 			}
 #endif	/* _HANDLE_ENLIST_IN_DTC_ */
 			CC_set_autocommit(conn, autocomm_on);
@@ -823,7 +824,7 @@ PGAPI_GetStmtOption(
 			{
 				char		option[64];
 
-				SC_set_error(stmt, STMT_NOT_IMPLEMENTED_ERROR, "Unknown statement option (Get)", NULL);
+				SC_set_error(stmt, STMT_NOT_IMPLEMENTED_ERROR, "Unknown statement option (Get)", func);
 				sprintf(option, "fOption=%d", fOption);
 				SC_log_error(func, option, stmt);
 				return SQL_ERROR;
