@@ -379,7 +379,9 @@ static BOOL CheckHasOids(StatementClass * stmt)
 	if (!stmt->ti || !stmt->ti[0])
 		return FALSE;
 	ti = stmt->ti[0];
-	sprintf(query, "select relhasoids, c.oid from pg_class c, pg_namespace n where relname = '%s' and nspname = '%s' and c.relnamespace = n.oid", SAFE_NAME(ti->table_name), SAFE_NAME(ti->schema_name));
+	snprintf(query, sizeof(query),
+			 "select relhasoids, c.oid from pg_class c, pg_namespace n where relname = '%s' and nspname = '%s' and c.relnamespace = n.oid",
+			 SAFE_NAME(ti->table_name), SAFE_NAME(ti->schema_name));
 	res = CC_send_query(conn, query, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
 	if (QR_command_maybe_successful(res))
 	{
@@ -738,8 +740,10 @@ COL_INFO **coli)
 				/*
 			  	 * We also have to check as follows.
 			  	 */
-				sprintf(token, "select nspname from pg_namespace n, pg_class c"
-						" where c.relnamespace=n.oid and c.oid='\"%s\"'::regclass", SAFE_NAME(table_name));
+				snprintf(token, sizeof(token),
+						 "select nspname from pg_namespace n, pg_class c"
+						 " where c.relnamespace=n.oid and c.oid='\"%s\"'::regclass",
+						 SAFE_NAME(table_name));
 				res = CC_send_query(conn, token, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
 				if (QR_command_maybe_successful(res))
 				{
