@@ -26,13 +26,15 @@
 #include <openssl/ssl.h>
 #endif /* USE_SSL */
 #endif /* NOT_USE_LIBPQ */
+#include "misc.h"
 #include "loadlib.h"
 
 #include "connection.h"
 
 #ifdef WIN32
 #include <time.h>
-#include <Wspiapi.h>
+#include <Wspiapi.h>	/* Ensure to support the backward-compatibility version
+			   of getaddrinfo() etc */
 #else
 #include <stdlib.h>
 #include <string.h>				/* for memset */
@@ -65,16 +67,6 @@ static void SOCK_set_error(SocketClass *s, int _no, const char *_msg)
 	mylog("(%d)%s ERRNO=%d\n", _no, _msg, gerrno);
 }
 
-void
-SOCK_clear_error(SocketClass *self)
-{
-	self->errornumber = 0;
-	if (NULL != self->_errormsg_)
-		free(self->_errormsg_);
-	self->_errormsg_ = NULL;
-}
-
-
 SocketClass *
 SOCK_Constructor(const ConnectionClass *conn)
 {
@@ -107,9 +99,15 @@ SOCK_Constructor(const ConnectionClass *conn)
 		rv->buffer_read_in = 0;
 
 		if (conn)
+		{
 			rv->buffer_size = conn->connInfo.drivers.socket_buffersize;
+			rv->keepalive = !conn->connInfo.disable_keepalive;
+		}
 		else
+		{
 			rv->buffer_size = globals.socket_buffersize;
+			rv->keepalive = TRUE;
+		}
 		rv->buffer_in = (UCHAR *) malloc(rv->buffer_size);
 		if (!rv->buffer_in)
 		{
@@ -187,11 +185,17 @@ SOCK_Destructor(SocketClass *self)
 	free(self);
 }
 
+#ifdef	WIN32
+static inet_pton_func inet_pton_ptr = NULL;
+static HMODULE ws2_hnd = NULL;
+#else
+static inet_pton_func inet_pton_ptr = inet_pton;
+#endif /* WIN32 */
+
 #if defined(_MSC_VER) && (_MSC_VER < 1300)
 static freeaddrinfo_func freeaddrinfo_ptr = NULL;
 static getaddrinfo_func getaddrinfo_ptr = NULL;
 static getnameinfo_func getnameinfo_ptr = NULL;
-static	HMODULE ws2_hnd = NULL;
 #else
 static freeaddrinfo_func freeaddrinfo_ptr = freeaddrinfo;
 static getaddrinfo_func getaddrinfo_ptr = getaddrinfo;
@@ -238,7 +242,19 @@ static BOOL format_sockerr(char *errmsg, size_t buflen, int errnum, const char *
 		snprintf(errmsg, buflen, "%s failed for [%s:%d] ", cmd, host, portno);
 	return ret;
 }
- 
+
+static int
+is_numeric_address(const char *hostname)
+{
+	if (inet_pton_ptr != NULL)
+	{
+		char unused[16];
+		return (inet_pton_ptr(AF_INET, hostname, &unused) ||
+			inet_pton_ptr(AF_INET6, hostname, &unused));
+	}
+	return (inet_addr(hostname) != INADDR_NONE);
+}
+
 char
 SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long timeout)
 {
@@ -253,9 +269,15 @@ SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long tim
 		return 0;
 	}
 
-#if defined(_MSC_VER) && (_MSC_VER < 1300)
+#ifdef	WIN32
 	if (ws2_hnd == NULL)
+	{
 		ws2_hnd = GetModuleHandle("ws2_32.dll");
+		if (inet_pton_ptr == NULL)
+			inet_pton_ptr = (inet_pton_func)GetProcAddress(ws2_hnd, "inet_pton");
+	} 
+#endif /* WIN32 */
+#if defined(_MSC_VER) && (_MSC_VER < 1300)
 	if (freeaddrinfo_ptr == NULL)
 		freeaddrinfo_ptr = (freeaddrinfo_func)GetProcAddress(ws2_hnd, "freeaddrinfo"); 
 	if (getaddrinfo_ptr == NULL)
@@ -279,8 +301,12 @@ SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long tim
 		rest.ai_socktype = SOCK_STREAM;
 		rest.ai_family = AF_UNSPEC;
 		snprintf(portstr, sizeof(portstr), "%d", port);
-		if (inet_addr(hostname) != INADDR_NONE)
-			rest.ai_flags |= AI_NUMERICHOST;	
+#ifdef	AI_NUMERICSERV
+		rest.ai_flags |= AI_NUMERICSERV;
+#endif /* AI_NUMERICSERV */
+		if (is_numeric_address(hostname))
+			/* don't resolve address in getaddrinfo() if not necessary */
+			rest.ai_flags |= AI_NUMERICHOST;
 		ret = getaddrinfo_ptr(hostname, portstr, &rest, &addrs);
 		if (ret || !addrs)
 		{
@@ -331,6 +357,21 @@ retry:
 		}
 	}
 #endif /* TCP_NODELAY */
+#ifdef	SO_KEEPALIVE
+	if (family != AF_UNIX && self->keepalive)
+	{
+		int i;
+		socklen_t	len;
+
+		i = 1;
+		len = sizeof(i);
+		if (setsockopt(self->socket, SOL_SOCKET, SO_KEEPALIVE, (char *) &i, len) < 0)
+		{
+			SOCK_set_error(self, SOCKET_COULD_NOT_CONNECT, "Could not set socket to SO_KEEPALIVE.");
+			goto cleanup;
+		}
+	}
+#endif /* SO_KEEPALIVE */
 #ifdef	WIN32
 	{
 		long	ioctlsocket_ret = 1;
