@@ -19,6 +19,7 @@
 #endif /* WIN_MULTITHREAD_SUPPORT */
 
 #include "statement.h"
+#include "misc.h" // strncpy_null
 
 #include "bind.h"
 #include "connection.h"
@@ -26,6 +27,7 @@
 #include "qresult.h"
 #include "convert.h"
 #include "environ.h"
+#include "loadlib.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -140,6 +142,14 @@ static const struct
 	,{
 		STMT_TYPE_EXPLAIN, "EXPLAIN"
 	}
+
+	/*
+	 * Special-commands that cannot be run in a transaction block. This isn't
+	 * as granular as it could be. VACUUM can never be run in a transaction
+	 * block, but some variants of REINDEX and CLUSTER can be. CHECKPOINT
+	 * doesn't throw an error if you do, but it cannot be rolled back so
+	 * there's no point in beginning a new transaction for it.
+	 */
 	,{
 		STMT_TYPE_SPECIAL, "VACUUM"
 	}
@@ -152,6 +162,7 @@ static const struct
 	,{
 		STMT_TYPE_SPECIAL, "CHECKPOINT"
 	}
+
 	,{
 		STMT_TYPE_WITH, "WITH"
 	}
@@ -664,7 +675,7 @@ SC_set_current_col(StatementClass *stmt, int col)
 }
 
 void
-SC_set_prepared(StatementClass *stmt, BOOL prepared)
+SC_set_prepared(StatementClass *stmt, int prepared)
 {
 	if (prepared == stmt->prepared)
 		;
@@ -672,6 +683,7 @@ SC_set_prepared(StatementClass *stmt, BOOL prepared)
 	{
 		ConnectionClass *conn = SC_get_conn(stmt);
 
+		ENTER_CONN_CS(conn);
 		if (conn && CONN_CONNECTED == conn->status)
 		{
 			if (CC_is_in_error_trans(conn))
@@ -688,6 +700,7 @@ SC_set_prepared(StatementClass *stmt, BOOL prepared)
 				QR_Destructor(res);
 			} 
 		}
+		LEAVE_CONN_CS(conn);
 	}
 	if (NOT_YET_PREPARED == prepared)
 		SC_set_planname(stmt, NULL);
@@ -956,7 +969,7 @@ SC_scanQueryAndCountParams(const char *query, const ConnectionClass *conn,
 		}
 		if (!multi && del_found)
 		{
-			if (!isspace(tchar))
+			if (!isspace((UCHAR) tchar))
 			{
 				multi = TRUE;
 				if (next_cmd)
@@ -1071,7 +1084,7 @@ SC_scanQueryAndCountParams(const char *query, const ConnectionClass *conn,
 					comment_level++;
 				}
 			}
-			if (!isspace(tchar))
+			if (!isspace((UCHAR) tchar))
 				bchar = tchar;
 		}
 	}
@@ -1114,11 +1127,11 @@ SC_pre_execute(StatementClass *self)
 			{
 				case NAMED_PARSE_REQUEST:
 				case PARSE_TO_EXEC_ONCE:
-					if (SQL_SUCCESS != prepareParameters(self, TRUE))
+					if (SQL_SUCCESS != prepareParameters(self))
 						return num_fields;
 					break;
 				case PARSE_REQ_FOR_INFO:
-					if (SQL_SUCCESS != prepareParameters(self, TRUE))
+					if (SQL_SUCCESS != prepareParameters(self))
 						return num_fields;
 					self->status = STMT_PREMATURE;
 					self->inaccurate_result = TRUE;
@@ -1672,7 +1685,7 @@ inolog("%s statement=%p res=%x ommitted=0\n", func, self, res);
 					switch (QR_get_rstatus(res))
 					{
 						case PORES_NO_MEMORY_ERROR:
-							SC_set_error(self, STMT_NO_MEMORY_ERROR, NULL, __FUNCTION__);
+							SC_set_error(self, STMT_NO_MEMORY_ERROR, "memory allocation error???", __FUNCTION__);
 							break;
 						case PORES_BAD_RESPONSE:
 							SC_set_error(self, STMT_COMMUNICATION_ERROR, "communication error occured", __FUNCTION__);
@@ -1895,15 +1908,22 @@ SC_execute(StatementClass *self)
 	    /* || SC_is_with_hold(self) thiw would lose the performance */
 		 ))
 		issue_begin = FALSE;
-	else
+	else if (self->statement_type == STMT_TYPE_SPECIAL)
 	{
-		switch (self->statement_type)
-		{
-			case STMT_TYPE_START:
-			case STMT_TYPE_SPECIAL:
-				issue_begin = FALSE;
-				break;
-		}
+		/*
+		 * Some utility commands like VACUUM cannot be run in a transaction
+		 * block, so don't begin one even if auto-commit mode is disabled.
+		 *
+		 * An application should never issue an explicit BEGIN when
+		 * auto-commit mode is disabled (probably not even when it's enabled,
+		 * actually). We used to also suppress the implicit BEGIN when the
+		 * statement was of STMT_TYPE_START type, ie. if the application
+		 * issued an explicit BEGIN, but that actually seems like a bad idea.
+		 * First of all, if you issue a BEGIN twice the backend will give a
+		 * warning which can be helpful to spot mistakes in the application
+		 * (because it shouldn't be doing that).
+		 */
+		issue_begin = FALSE;
 	}
 	if (issue_begin)
 	{
@@ -2449,6 +2469,10 @@ RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
 {
 	BOOL	ret = TRUE;
 
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+	if (conn->asdum)
+		CALL_IsolateDtcConn(conn, TRUE);
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
 	if (SC_accessed_db(stmt))
 		return TRUE;
 	if (SQL_ERROR == SetStatementSvp(stmt))
@@ -2459,10 +2483,16 @@ RequestStart(StatementClass *stmt, ConnectionClass *conn, const char *func)
 		SC_set_error(stmt, STMT_INTERNAL_ERROR, emsg, func);
 		return FALSE;
 	}
-	if (!CC_is_in_trans(conn) && CC_loves_visible_trans(conn))
+
+	/*
+	 * In auto-commit mode, begin a new transaction implicitly if no
+	 * transaction is in progress yet. However, some special statements like
+	 * VACUUM and CLUSTER cannot be run in a transaction block.
+	 */
+	if (!CC_is_in_trans(conn) && CC_loves_visible_trans(conn) &&
+		stmt->statement_type != STMT_TYPE_SPECIAL)
 	{
-		if (ret = CC_begin(conn), !ret)
-			return ret;
+		ret = CC_begin(conn);
 	}
 	return ret;
 }
